use std::collections::HashMap;
use std::sync::Arc;

use serde::Deserialize;

use deckster_shared::path::{KeyPosition, KnobPosition};
use deckster_shared::style::KeyStyle;

use crate::model::geometry::UIntVec2;
use crate::modes;

#[derive(Debug, Deserialize)]
pub struct File {
    pub id: Option<String>,
    pub scrolling: Option<ScrollingConfig>,
    pub keys: HashMap<KeyPosition, Key>,
}

#[derive(Debug)]
pub struct Page {
    pub id: String,
    pub scrolling: Option<ScrollingConfig>,
    pub keys: HashMap<KeyPosition, Arc<Key>>,
}

#[derive(Debug, Deserialize)]
pub struct ScrollingConfig {
    pub knob: Option<KnobPosition>,
    pub key: Option<ScrollingKeysConfig>,
    pub delta: u8,
    #[serde(default)]
    pub axis: ScrollingConfigAxis,
}

#[derive(Debug, Deserialize)]
pub struct ScrollingKeysConfig {
    pub previous: UIntVec2,
    pub next: UIntVec2,
}

#[derive(Debug, Default, Eq, PartialEq, Hash, Deserialize)]
#[serde(rename_all = "kebab-case")]
pub enum ScrollingConfigAxis {
    #[default]
    Vertical,
    Horizontal,
}

#[derive(Debug, Deserialize)]
pub struct Key {
    #[serde(default, flatten)]
    pub base_style: KeyStyle,

    #[serde(default)]
    pub mode: KeyModes,
}

#[allow(non_snake_case)]
#[derive(Debug, Default, Deserialize)]
pub struct KeyModes {
    pub command: Option<Arc<modes::key::command::Config>>,
    pub home_assistant__switch: Option<Arc<modes::key::home_assistant::SwitchConfig>>,
    pub home_assistant__button: Option<Arc<modes::key::home_assistant::ButtonConfig>>,
    pub playerctl__button: Option<Arc<modes::key::playerctl::ButtonConfig>>,
    pub playerctl__shuffle: Option<Arc<modes::key::playerctl::ShuffleConfig>>,
    pub playerctl__loop: Option<Arc<modes::key::playerctl::LoopConfig>>,
}
