use std::sync::Arc;

use tokio::sync::broadcast;

use deckster_shared::handler_communication::HandlerCommand;
use deckster_shared::handler_communication::KnobEvent;
use deckster_shared::path::KnobPath;

use crate::model;

pub mod audio_volume;

pub fn start_handlers(
    knobs: impl Iterator<Item = (KnobPath, Arc<model::knob_page::Knob>)>,
    events: broadcast::Sender<(KnobPath, KnobEvent)>,
    commands: flume::Sender<HandlerCommand>,
) {
    for (path, config) in knobs {
        if let Some(c) = &config.mode.audio_volume {
            tokio::spawn(audio_volume::handle(path.clone(), Arc::clone(c), events.subscribe(), commands.clone()));
        }
    }
}
