use serde::{Deserialize, Serialize};

use crate::path::{KeyPath, KnobPath};
use crate::style::{KeyStyle, KnobStyle};

#[derive(Debug, Eq, PartialEq, Copy, Clone, Serialize, Deserialize)]
pub enum RotationDirection {
    Clockwise,
    Counterclockwise,
}

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub enum KnobEvent {
    Press,
    ButtonDown,
    ButtonUp,
    Rotate { direction: RotationDirection },
    VisibilityChange { is_visible: bool },
}

#[derive(Debug, PartialEq, Clone, Copy, Serialize, Deserialize)]
pub enum KeyTouchEventKind {
    Start,
    Move,
    End,
}

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub enum KeyEvent {
    Press,
    Touch { touch_id: u8, x: u16, y: u16, kind: KeyTouchEventKind },
    VisibilityChange { is_visible: bool },
}

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
#[serde(tag = "type", rename_all = "kebab-case")]
pub enum HandlerEvent {
    Knob { path: KnobPath, event: KnobEvent },
    Key { path: KeyPath, event: KeyEvent },
}

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
#[serde(tag = "command", rename_all = "kebab-case")]
pub enum HandlerCommand {
    SetActivePages { key_page_id: String, knob_page_id: String },
    SetKeyStyle { path: KeyPath, value: Option<KeyStyle> },
    SetKnobStyle { path: KnobPath, value: Option<KnobStyle> },
    SetKnobValue { path: KnobPath, value: Option<f32> },
}

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub struct InitialHandlerMessage<KeyConfig: Clone, KnobConfig: Clone> {
    key_configs: im::HashMap<KeyPath, (String, KeyConfig)>,
    knob_configs: im::HashMap<KnobPath, (String, KnobConfig)>,
}
